ul.nav ul {
  list-style:none;
  margin: 0;
  padding: 0 0 0 25px;
}

#downloader_application form {
  margin-bottom: 10px;
}

#downloader_application ul {
  list-style-type: none;
}

.browser_support th {
  border-bottom-width: 3px !important;
}

.support_ie {border-bottom-color: #0275BA !important;}
.support_ff {border-bottom-color: #DF7215 !important;}
.support_sf {border-bottom-color: #43B3E9 !important;}
.support_cr {border-bottom-color: #39B642 !important;}
.support_op {border-bottom-color: #C42122 !important;}
.support_nd {border-bottom-color: #8CC84B !important;}

.show-example {
    padding: 10px;
    border: 1px solid #ccc;
    border-radius: 4px;
    margin: 0 0 10px;
}

.tab-pane > figure.highlight > pre, .tab-pane > .show-example {
    border-radius: 0 0 4px 4px;
    border-top: 0px;
}
